<?php
/*+**********************************************************************************
 * The contents of this file are subject to the vtiger CRM Public License Version 1.0
 * ("License"); You may not use this file except in compliance with the License
 * The Original Code is:  vtiger CRM Open Source
 * The Initial Developer of the Original Code is vtiger.
 * Portions created by vtiger are Copyright (C) vtiger.
 * All Rights Reserved.
 ************************************************************************************/

set_include_path ( 'include/pdb' . PATH_SEPARATOR . get_include_path() );

include_once 'PDB.php';
include_once 'adodb/adodb.inc.php';
include_once 'adodb/adodb-xmlschema.inc.php';

/**
 * Class that provides interfacing between both PDO & ADODB libs.
 * to continue with backward compatibility until platform switches completely.
 */
class PDO_ADODB_Connector {
	var $debug;
	
	protected $fetchMode = PDO::FETCH_ASSOC;
		
	protected $pdbConnection = null;
	protected $adodbConnection = null;
	protected $logSQL = false;
	
	protected $options = array(
		'dbtype' => '',
		'host'   => '',
		'user'   => '',
		'password'=> '',
		'dbname' => ''
	);
	
	function __construct($dbtype) {
		$this->options['dbtype'] = $dbtype;
	}
	
	function __destruct() {
		$this->disconnect();
	}
	
	function PConnect($hostport, $user, $password, $dbname) {
		$this->options['hostport'] = $hostport;
		list($host, $port)  = explode(':', $hostport);
		$this->options['user'] = $user;
		$this->options['password']= $password;
		$this->options['dbname'] = $dbname;
		
		$dbtype = $this->options['dbtype'];
		
		try {
			$this->pdbConnection = PDB::singleton("$dbtype:host=$host;port=$port;dbname=$dbname", $user, $password);
		} catch (Exception $e) {
			die('Cannot establish connection with database!');
		}
		
		$this->pdbConnection->fetchMode = $this->fetchMode;
	}
	
	function LogSQL($value) {
		$this->logSQL = $value;
	}

	function adodbInstance() {
		if (!$this->adodbConnection) {
			$this->adodbConnection = ADONewConnection($this->options['dbtype']);
			$this->adodbConnection->PConnect($this->options['hostport'], $this->options['user'], $this->options['password'], $this->options['dbname']);
			$this->adodbConnection->LogSQL($this->logSQL);
		}
		return $this->adodbConnection;
	}
	
	function SetFetchMode($mode) {
		// TO BE HANDLED
	}
	
	function getError() {
		// TO BE HANDLED		
	}
	
	function setOption($option) {
		// TO BE HANDLED		
	}
	
	function StartTrans() {	
		// TO BE HANDLED		
	}
	function HasFailedTrans() {
		// TO BE HANDLED
		return false;
	}	
	function CompleteTrans() {
		// TO BE HANDLED		
	}
	
	function ErrorMsg() {
		$errorInfo = $this->pdbConnection->errorInfo();
		return $errorInfo? $errorInfo[0]: false;
	}
	
	function ErrorNo() {
		return $this->pdbConnection->errorCode();
	}
	
	function Execute($query, $params=false) {		
		// Turn false/null to array()
		if (!is_array($params)) $params = array();
		
		$result = $this->pdbConnection->query($query, $params);
		$rows   = array();		
		if ($result) {
			if ($result->rowCount()) {
				try {
					$rows = $result->fetchAll($this->fetchMode);
				} catch (Exception $e) {
					// Ignore ? - Attempt to fetch for INSERT/DELETE/UPDATE puts us here.
				}
			}
	        $result->closeCursor();			
		}		
		return new PDO_ADODB_Result($rows);
	}
	
	function Quote($input) {
		return $this->pdbConnection->quote($input);
	}
	
	function UpdateBlob($table, $column, $data, $id) {
		// TO BE HANDLED		
	}
	
	function UpdateBlobFile($table, $column, $filename, $id) {		
		// TO BE HANDLED		
	}
	
	function SelectLimit($sql, $count=-1, $start=-1, $params=false, $secs=0) {
		$sql = rtrim($sql, ';');
		
		$limit = '';
		if ($start !== -1 && $count !== -1) {
			$limit .= $start.','.$count;
		} else if ($start !== -1) {
			$limit .= $start;
		} else if ($count !== -1) {
			$limit .= $count;
		}
		if ($limit) {
			$sql .= " LIMIT {$limit}";
		}
		
		return $this->Execute($sql, $params);		
	}
	
	function GetOne($sql) {
		return $this->pdbConnection->getOne($sql);
	}
	
	function qstr($input) {
		return $this->Quote($input);
	}
	
	function disconnect() {
		if ($this->pdbConnection) {
			$this->pdbConnection->disconnect();
		}
		if ($this->adodbConnection) {
			$this->adodbConnection->disconnect();
		}		
	}
	
	function MetaColumns($table) {
		return $this->adodbInstance()->MetaColumns($table);	
	}
	
	function DBTimeStamp($datetime) {
		return $this->adodbInstance()->DBTimeStamp($datetime);
	}
	
	function GenID($seqName='_seq', $startId=1) {
		return $this->adodbInstance()->GenID($seqName, $startId);
	}
	
	function MetaTables($ttype=false, $showSchema=false) {
		return $this->adodbInstance()->Metatables($ttype, $showSchema);
	}
	
	function Insert_ID() {
		return $this->pdbInstance->lastInsertId();
	}	
}

class PDO_ADODB_Result {
	var $EOF = true;
	
	protected $resultSet;
	protected $size;
	protected $currentIndex;
	
	function __construct(&$rs) {
		$this->resultSet =& $rs;
		$this->size = count($this->resultSet);
		$this->MoveFirst();
	}
	
	function __toString() {
		return 'PDO_ADODB_Result';
	}
	
	function FieldCount() {	
		// TO BE HANDLED
	}
	
	function RecordCount() {
		return $this->size;
	}
	
	function FetchField($index) {
		$row = $this->FetchRow();
		if ($row) {
			$values = array_values($row);
			return $values[$index];
		}
		return null;
	}
	
	function FetchRow() {
		$ret = null;
		if ($this->currentIndex < $this->size) {
			$ret =& $this->resultSet[$this->currentIndex];
			$this->MoveNext();			
		}
		return $ret;
	}
	
	function GetRowAssoc($toUpper = false) {
		// TO HANDLE $toUpper (http://phplens.com/adodb/reference.functions.getrowassoc.html)
		$ret = null;
		if ($this->currentIndex < $this->size) {
			$ret =& $this->resultSet[$this->currentIndex];
		}
		return $ret;
	}
	
	protected function updateEOF() {
		$this->EOF = ($this->currentIndex >= $this->size);		
	}
	
	function MoveFirst() {
		$this->Move(0);	
	}
	
	function CurrentRow() {
		return $this->currentIndex;
	}
	
	function MoveNext() {
		$this->Move();
	}
	
	function Move($row=false) {
		if ($row === false) $this->currentIndex++;
		else $this->currentIndex = $row;
		
		$this->updateEOF();		
	}
}